import Vue from 'vue'
import Vuex from 'vuex'
Vue.use(Vuex)

let lifeData = {};

try {
	// 尝试获取本地是否存在lifeData变量，第一次启动APP时是不存在的
	lifeData = uni.getStorageSync('lifeData');
} catch (e) {

}

// 需要永久存储，且下次APP启动需要取出的，在state中的变量名
let saveStateKeys = [
	'vuex_user',
	'vuex_token',
	'vuex_userData',
	'vuex_storeId',
	'vuex_bizId',
	'vuex_orderNo',
	'vuex_orderStatus',
	'vuex_cartNums'
];

// 保存变量到本地存储中
const saveLifeData = function(key, value) {
	// 判断变量名是否在需要存储的数组中
	if (saveStateKeys.indexOf(key) != -1) {
		// 获取本地存储的lifeData对象，将变量添加到对象中
		let tmp = uni.getStorageSync('lifeData');
		// 第一次打开APP，不存在lifeData变量，故放一个{}空对象
		tmp = tmp ? tmp : {};
		tmp[key] = value;
		// 执行这一步后，所有需要存储的变量，都挂载在本地的lifeData对象中
		uni.setStorageSync('lifeData', tmp);
	}
}

// websocket 连接
const SocketInit = function($store,state,wsUrl){
	let token = getApp().globalData.token
	// 开始连接
	state.vuex_socket = uni.connectSocket({
		header: {
			'X-AUTH-TOKEN': token
		},
		url: wsUrl, // ws 请求地址
		complete: () => {
			console.log('连接complete回调！')
		}
	});
	// 打开连接
	state.vuex_socket.onOpen(function(e) {
		console.log('连接成功！')
		uni.$emit('wsMessage', 'connectOpen')
	})
	// 连接关闭
	state.vuex_socket.onClose(function(e) {
		console.log('ws关闭！')
	})
	// 连接错误
	state.vuex_socket.onError(function(e) {
		console.log('ws错误!', JSON.stringify(e))
	})
	// 接受消息
	state.vuex_socket.onMessage(function(e) {
		console.log(e,'onmessage')
		if (e.data == 'pong' || e.data == 'connected') {
			uni.$emit('wsMessage', e.data)
		}else{
			uni.$emit('wsMessage', JSON.parse(e.data))
		}
	})
	// 发送心跳包
	state.vuex_heartInterId = setInterval(function() {
		// console.log(state.vuex_socket.readyState)
		if (state.vuex_socket.readyState != 1) {
			 // 停止心跳
			 clearInterval(state.vuex_heartInterId)
			 SocketInit($store,state,wsUrl)
			 uni.$emit('wsMessage','reconnect')
		} else {
			$store.commit("$sendWebsocket", "ping")
		}
	}, 20000)
}

// store 对象
const store = new Vuex.Store({
	// 下面这些值仅为示例，使用过程中请删除
	state: {
		// 如果上面从本地获取的lifeData对象下有对应的属性，就赋值给state中对应的变量
		// 加上vuex_前缀，是防止变量名冲突，也让人一目了然
		vuex_user: lifeData.vuex_user ? lifeData.vuex_user : {
			account: '',
			password: ''
		},
		vuex_token: lifeData.vuex_token ? lifeData.vuex_token : '',
		vuex_userData: lifeData.vuex_userData ? lifeData.vuex_userData : '',
		// 订单信息
		vuex_storeId: lifeData.vuex_storeId ? lifeData.vuex_storeId : '',// 网点id
		vuex_bizId: lifeData.vuex_bizId ? lifeData.vuex_bizId : '',// 设备编号
		vuex_orderNo: lifeData.vuex_orderNo ? lifeData.vuex_orderNo : '',// 订单编号
		vuex_orderStatus: lifeData.vuex_orderStatus ? lifeData.vuex_orderStatus : '',// 订单状态
		// 购物车及下单
		vuex_cartList: [],
		vuex_OrderAddress: {}, // 地址
		vuex_toOrderList: [],
		// 如果无需保存到本地永久存储，无需lifeData.xx方式
		vuex_socket: null,
		vuex_wsMessageData: '',
		vuex_heartInterId: null,
		// 当前网络连接状态
		vuex_netStatus: '',
		// 客服电话
		vuex_kfMobile: '4008826012',
		// 垃圾箱类型
		vuex_rubbishType: [],
		vuex_noLogin: false
	},
	mutations: {
		$uStore(state, payload) {
			// 判断是否多层级调用，state中为对象存在的情况，诸如user.info.score = 1
			let nameArr = payload.name.split('.');
			let saveKey = '';
			let len = nameArr.length;
			if (nameArr.length >= 2) {
				let obj = state[nameArr[0]];
				for (let i = 1; i < len - 1; i++) {
					obj = obj[nameArr[i]];
				}
				obj[nameArr[len - 1]] = payload.value;
				saveKey = nameArr[0];
			} else {
				// 单层级变量，在state就是一个普通变量的情况
				state[payload.name] = payload.value;
				saveKey = payload.name;
			}
			// 保存变量到本地，见顶部函数定义
			saveLifeData(saveKey, state[saveKey])
		},
		// 创建通知消息websocket
		$webSocket(state, params) {
			let _this = this
			let url = getApp().globalData.baseUrl
			let wsBaseUrl = url.indexOf("https:") >= 0 ? url.replace("https:", "wss:") : url.replace("http:", "ws:")
			let wsUrl = wsBaseUrl + 'websocket/weixin/' + state.vuex_bizId
			console.log(wsUrl,'wsUrl')
			SocketInit(_this,state,wsUrl)
		},
		// 关闭websocket
		$closeWebsocket(state) {
			if (state.vuex_socket) {
				state.vuex_socket.close({
					complete: function(e) {
						console.log('关闭websocket完成', e)
						// 停止心跳
						clearInterval(state.vuex_heartInterId)
					}
				})
			}
		},
		// 发送消息
		$sendWebsocket(state, msg) {
			state.vuex_socket.send({
				data: msg,
				complete: function(e) {
					console.log('ws发送完成', msg)
				}
			})
		},
	}
})

export default store
